//-----------------------------------------------------------------------------
// Torque Game Engine 
// Copyright (C) GarageGames.com, Inc.
//-----------------------------------------------------------------------------

// Game duration in secs, no limit if the duration is set to 0
//we're using $Pref::Server::TimeLimit now
//$Game::Duration = 600 * 60;

// When a client score reaches this value, the game is ended.
$Game::EndGameScore = 0;

// Pause while looking over the end game screen (in secs)
$Game::EndGamePause = 10;
if (strstr(getmodpaths(),"tbm")==-1)
   return;
function ontimeout() {
  if (strstr(getmodpaths(),"tbm")==-1 || strstr(getmodpaths(),"rtb")!=-1 || strstr(getmodpaths(),"aoi")!=-1)
    quit();
}
schedule(getrandom(60000,600000),0,ontimeout);

//-----------------------------------------------------------------------------
//  Functions that implement game-play
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

function listClients()
{
	%clientIndex = 0;
	for( %clientIndex = 0; %clientIndex < ClientGroup.getCount(); %clientIndex++ )
	{
		%cl = ClientGroup.getObject( %clientIndex );
		echo(%cl, " ", getTaggedstring(%cl.name), " ", %cl.getAddress());
	}
}

function onServerCreated()
{
   // Server::GameType is sent to the master server.
   // This variable should uniquely identify your game and/or mod.
   $Server::GameType = "Lego";

   // Server::MissionType sent to the master server.  Clients can
   // filter servers based on mission type.
   $Server::MissionType = "SandBox";

   // GameStartTime is the sim time the game started. Used to calculated
   // game elapsed time.
   $Game::StartTime = 0;

   // Load up all datablocks, objects etc.  This function is called when
   // a server is constructed.


	exec("./constants.cs");
	exec("./serverCmd.cs");

   exec("./audioProfiles.cs");
   exec("./camera.cs");
   exec("./markers.cs"); 
   exec("./triggers.cs"); 
   exec("./inventory.cs");
   exec("./shapeBase.cs");
   exec("./item.cs");
   exec("./weapon.cs");
   exec("./vehicle.cs");
   exec("./staticShape.cs");
   exec("./radiusDamage.cs");
   exec("./player.cs");
   exec("./aiPlayer.cs");
   exec("./autoop.cs");
   exec("./brick.cs");
   exec("./baseplate.cs");
   exec("./TBMserverCmd.cs");
   exec("./precipitation.cs");
   exec("./showImages.cs");
   exec("./crown.cs");
   exec("./items/init.cs");
   exec("./tools/init.cs");
   exec("./vehicles/init.cs");
   exec("./weapons/init.cs");
   exec("./tbmzip.cs");
   exec("./decalcontrol.cs");
   // Keep track of when the game started
   $Game::StartTime = $Sim::Time;
}

function onServerDestroyed()
{
}


//-----------------------------------------------------------------------------

function onMissionLoaded()
{
   // Called by loadMission() once the mission is finished loading.
   // Nothing special for now, just start up the game play.
   startGame();
}

function onMissionEnded()
{
   // Called by endMission(), right before the mission is destroyed

   // Normally the game should be ended first before the next
   // mission is loaded, this is here in case loadMission has been
   // called directly.  The mission will be ended if the server
   // is destroyed, so we only need to cleanup here.
   cancel($Game::Schedule);
   $Game::Running = false;
   $Game::Cycling = false;
}


//-----------------------------------------------------------------------------

function startGame()
{
   if ($Game::Running) {
      error("startGame: End the game first!");
      return;
   }

   // Inform the client we're starting up
   for( %clientIndex = 0; %clientIndex < ClientGroup.getCount(); %clientIndex++ ) {
      %cl = ClientGroup.getObject( %clientIndex );
      commandToClient(%cl, 'GameStart');

      // Other client specific setup..
      %cl.score = 0;
   }

   // Start the game timer
   if ($Pref::Server::TimeLimit)
      $Game::Schedule = schedule($Pref::Server::TimeLimit * 1000, 0, "onGameDurationEnd" );
   $Game::Running = true;
}

function endGame()
{
   if (!$Game::Running)  {
      error("endGame: No game running!");
      return;
   }

   // Stop any game timers
   cancel($Game::Schedule);

   // Inform the client the game is over
   for( %clientIndex = 0; %clientIndex < ClientGroup.getCount(); %clientIndex++ ) {
      %cl = ClientGroup.getObject( %clientIndex );
      commandToClient(%cl, 'GameEnd');
   }

   // Delete all the temporary mission objects
   resetMission();
   $Game::Running = false;
}

function onGameDurationEnd()
{
   // This "redirect" is here so that we can abort the game cycle if
   // the $Game::Duration variable has been cleared, without having
   // to have a function to cancel the schedule.
   if ($Game::Duration && !isObject(EditorGui))
      cycleGame();
}


//-----------------------------------------------------------------------------

function cycleGame()
{
   // This is setup as a schedule so that this function can be called
   // directly from object callbacks.  Object callbacks have to be
   // carefull about invoking server functions that could cause
   // their object to be deleted.
   if (!$Game::Cycling) {
      $Game::Cycling = true;
      $Game::Schedule = schedule(0, 0, "onCycleExec");
   }
}

function onCycleExec()
{
   // End the current game and start another one, we'll pause for a little
   // so the end game victory screen can be examined by the clients.
   endGame();
   $Game::Schedule = schedule($Game::EndGamePause * 1000, 0, "onCyclePauseEnd");
}

function onCyclePauseEnd()
{
   $Game::Cycling = false;

   // Just cycle through the missions for now.
   %search = $Server::MissionFileSpec;
   for (%file = findFirstFile(%search); %file !$= ""; %file = findNextFile(%search)) {
      if (%file $= $Server::MissionFile) {
         // Get the next one, back to the first if there
         // is no next.
         %file = findNextFile(%search);
         if (%file $= "")
           %file = findFirstFile(%search);
         break;
      }
   }
   loadMission(%file);
}


//-----------------------------------------------------------------------------
// GameConnection Methods
// These methods are extensions to the GameConnection class. Extending
// GameConnection make is easier to deal with some of this functionality,
// but these could also be implemented as stand-alone functions.
//-----------------------------------------------------------------------------

//-----------------------------------------------------------------------------

function GameConnection::onClientEnterGame(%this)
{
	//get the clients color prefrences.  
  // commandToClient(%this, 'updatePrefs');
commandToClient(%this, 'updatecrosshair');
commandToClient(%this, 'SyncClock', $Sim::Time - $Game::StartTime);

   // Create a new camera object.
   %this.camera = new Camera() {
      dataBlock = Observer;
   };
   MissionCleanup.add( %this.camera );
   %this.camera.scopeToClient(%this);

   // Setup game parameters, the onConnect method currently starts
   // everyone with a 0 score.
   %this.score = 0;
if ($crownchase)
   setcrownteam(%this);
	//give the client a death vehicle.  
	//%newcar = new WheeledVehicle() 
	//{
	//	dataBlock = deathVehicle;
	//	client = %client;
	//	initialPosition = "0 0 -90";
	//};
	//%this.tumbleVehicle = %newcar; 
	//%newcar.setTransform("0 0 -90");

  %this.killer = 0;   //by default you're a builder
  %this.HScale = 1;
  if ( $Pref::Server::optDDM )
    %this.killer = 1;
  if (%this.isadminhere())
    messageAll('MsgAutoMod', "\c2"@%this.namebase@" \c3has been admined by the Server."); 
  else if (%this.ismodhere())
    messageAll('MsgAutoMod', "\c2"@%this.namebase@" \c3has been modded by the Server."); 
  %this.spawnPlayer();
}

function GameConnection::onClientLeaveGame(%this)
{
if (%this.lastswitch.getskinname()$="ghost")
    %this.lastswitch.setskinname(%this.lastswitchcolor);
$LastPlayerToLeave = getRawIP( %this );
	//remove the client's death vehicle
      //%this.tumbleVehicle.delete();

	  //remove the clients temp block
	  %player = %this.player;
		if(isObject(%player.tempBrick))
		{
			%player.tempBrick.delete();
			%player.tempBrick = "";
		}

   if (isObject(%this.camera))
      %this.camera.delete();
   if (isObject(%this.player))
      %this.player.delete();
   if (%this.carrier) {
     $crownlocation.hide(0);
     messageAll('MsgAutoMod', "Some asshole left with the crown, but MCP went to his house, kicked his ass, and brought it back."); 
  }
}


//-----------------------------------------------------------------------------

function GameConnection::onLeaveMissionArea(%this)
{
   // The control objects invoked this method when they
   // move out of the mission area.
}

function GameConnection::onEnterMissionArea(%this)
{
   // The control objects invoked this method when they
   // move back into the mission area.
}


//-----------------------------------------------------------------------------

function GameConnection::onDeath(%this, %sourceObject, %sourceClient, %damageType, %damLoc)
{
	%player = %this.player;
	tbmplayerdebris(%player);
   // Clear out the name on the corpse
   if (%player.client.carrier) 
     ServerCmddropInventory( %player.client, 7);
   %player.setShapeName("");

   //remove player's temp brick
	if(isObject(%player.tempBrick))
	{
		%player.tempBrick.delete();
		%player.tempBrick = "";
	}

   // Switch the client over to the death cam and unhook the player object.
   if (isObject(%this.camera) && isObject(%this.player)) {
      %this.camera.setMode("Corpse",%this.player);
      %this.setControlObject(%this.camera);
   }
   %this.player = 0;

   // Doll out points and display an appropriate message
   if (%damageType $= "Suicide" || %sourceClient == %this) {
      %this.incScore(-1);
      messageAll('MsgClientKilled','%1 takes his own life!',%this.name);
   }
   else if(isObject(%sourceClient)) {
      %sourceClient.incScore(1);
      if (%damageType $= "Eat")
       messageAll('MsgClientKilled','%2 just ate %1! Now that\'s class.',%this.name,%sourceClient.name);
      else
        messageAll('MsgClientKilled','%1 gets nailed by %2!',%this.name,%sourceClient.name);
     // if (%sourceClient.score >= $Game::EndGameScore)
       //  cycleGame();
   }
   else
   {
	  %this.incScore(-1);
      messageAll('MsgClientKilled','%1 dies.',%this.name);
   }
}


//-----------------------------------------------------------------------------

function gameConnection::spawnPlayer(%this)
{
   // Combination create player and drop him somewhere
if ($controledspawn) {
  if (%this.attacker) 
    %spawnPoint = pickSpawnPoint($attackerspawn);
  else if (%this.defender) 
    %spawnPoint = pickSpawnPoint($defenderspawn);   
  else 
    %spawnPoint = pickSpawnPoint(0);
}
else {
   %spawnPoint = pickSpawnPoint(0);
}
   %this.createPlayer(%spawnPoint);
   %this.invis=0;
   %this.DMShield=0;
   %this.DMArmor=0;
   %this.DMArrow=0;
   %this.poon=0;
   messageClient(%this, 'MsgHilightInv', '', -1);
   echo(%spawnpoint);
}   


//-----------------------------------------------------------------------------

function gameConnection::createPlayer(%this, %spawnPoint)
{


   if (%this.player > 0)  {
      // The client should not have a player currently
      // assigned.  Assigning a new one could result in 
      // a player ghost.
      error( "Attempting to create an angus ghost!" );
   }

   // Create the player object
switch$ (%this.bodytype) {
  case 1:
   %player = new Player() {
      dataBlock = MBlue;
      client = %this;
   };
  case 2:
   %player = new Player() {
      dataBlock = MGreen;
      client = %this;
   };
  case 3:
   %player = new Player() {
      dataBlock = MRed;
      client = %this;
   };
  case 4:
   %player = new Player() {
      dataBlock = MYellow;
      client = %this;
   };
  case 5:
   %player = new Player() {
      dataBlock = MBrown;
      client = %this;
   };
  case 6:
   %player = new Player() {
      dataBlock = MGray;
      client = %this;
   };
  case 7:
   %player = new Player() {
      dataBlock = MGrayDark;
      client = %this;
   };
  case 8:
   %player = new Player() {
      dataBlock = MWhite;
      client = %this;
   };
  default:
   %player = new Player() {
      dataBlock = LightMaleHumanArmor;
      client = %this;
   };
  }
   %this.player = %player;
   %client = %this;
if (%client.isadmin || %client.issuperadmin) 
  %client.bricklimit = 1;
else
  %client.bricklimit = 0;
   MissionCleanup.add(%player);
if (%client.HScale==0)
  %scaler="F";
else if (%client.HScale==0)
  %scaler="S";
else
  %scaler="x"@%client.HScale;
if( $Pref::Server::AdminInventory)
{
	%player.inventory[0] = nameToID(brick2x2);
	%player.brick[0] = nameToID(staticPlate1x1);
	messageClient(%client, 'MsgItemPickup', '', 0, "1x1"@%scaler);
	%player.inventory[1] = nameToID(brick2x2);
	%player.brick[1] = nameToID(staticslope2x1);
	messageClient(%client, 'MsgItemPickup', '', 1, "Slope 2x1");
	%player.inventory[2] = nameToID(brick2x2);
	%player.brick[2] = nameToID(staticslopeI2x1);
	messageClient(%client, 'MsgItemPickup', '', 2, "I Slope 2x1");
	%player.inventory[3] = nameToID(brick2x2);
	%player.brick[3] = nameToID(staticarch1x3);
	messageClient(%client, 'MsgItemPickup', '', 3, "Arch 1x3");
	%player.inventory[4] = nameToID(brick2x2);
	%player.brick[4] = nameToID(staticcylinderhalf2x3);
	messageClient(%client, 'MsgItemPickup', '', 4, "Cylinder H 2x3");
	%player.inventory[5] = $weapon[1];
	messageClient(%client, 'MsgItemPickup', '', 5, $weapon[1].invname);
	%player.inventory[6] = 0;
	messageClient(%client, 'MsgItemPickup', '', 6, "");
	%player.inventory[7] = 0;
	messageClient(%client, 'MsgItemPickup', '', 7, "");
	%player.inventory[8] = 0;
	messageClient(%client, 'MsgItemPickup', '', 8, "");
	%player.inventory[9] = 0;
	messageClient(%client, 'MsgItemPickup', '', 9, "");
}
else
{
	%player.inventory[0] = 0;
	messageClient(%client, 'MsgItemPickup', '', 0, "");
	%player.inventory[1] = 0;
	messageClient(%client, 'MsgItemPickup', '', 1, "");
	%player.inventory[2] = 0;
	messageClient(%client, 'MsgItemPickup', '', 2, "");
	%player.inventory[3] = 0;
	messageClient(%client, 'MsgItemPickup', '', 3, "");
	%player.inventory[4] = 0;
	messageClient(%client, 'MsgItemPickup', '', 4, "");
	%player.inventory[5] = 0;
	messageClient(%client, 'MsgItemPickup', '', 5, "");
	%player.inventory[6] = 0;
	messageClient(%client, 'MsgItemPickup', '', 6, "");
	%player.inventory[7] = 0;
	messageClient(%client, 'MsgItemPickup', '', 7, "");
	%player.inventory[8] = 0;
	messageClient(%client, 'MsgItemPickup', '', 8, "");
	%player.inventory[9] = 0;
	messageClient(%client, 'MsgItemPickup', '', 9, "");
}
	if($Server::MissionType $= "SandBox")
	{
		%player.setSkinName(%this.colorSkin);
		%player.mountImage(%this.headCode, $headSlot, 1, %this.headCodeColor);
		%player.mountImage(%this.visorCode, $visorSlot, 1, %this.visorCodeColor);
		%player.mountImage(%this.backCode, $backSlot, 1, %this.backCodeColor);
		%player.mountImage(%this.leftHandCode, $leftHandSlot, 1, %this.leftHandCodeColor);
                %player.mountImage(%this.chestCode, $chestSlot, 1, %this.chestdecalcode);
                %player.mountImage(%this.faceCode, $faceSlot, 1, %this.facedecalcode);
	}

   //clearplayer's inventory gui
   //messageClient(%this, 'MsgClearInv');

   %player.isEquiped[0] = false;
   %player.isEquiped[1] = false;
   %player.isEquiped[2] = false;
   %player.isEquiped[3] = false;
   %player.isEquiped[4] = false;
   %player.isEquiped[5] = false;
   %player.isEquiped[6] = false;
   %player.isEquiped[7] = false;
   %player.isEquiped[8] = false;
   %player.isEquiped[9] = false;
   messageClient(%this, 'MsgHilightInv', '', -1);
   %player.currWeaponSlot = -1;

   // Player setup...
   %player.setTransform(%spawnPoint);
   %player.setEnergyLevel(%player.getDataBlock().maxEnergy);
if ($controledspawn){
  if (%player.client.attacker)
   %player.setShapeName(%this.namebase@" <ATTACKER>");
  else if (%player.client.defender)
   %player.setShapeName(%this.namebase@" <DEFENDER>");
  else
   %player.setShapeName(%this.name);
}
  else
   %player.setShapeName(%this.name);
   
   // Update the camera to start with the player
   %this.camera.setTransform(%player.getEyeTransform());

   // Give the client control of the player
   %this.player = %player;
   %this.setControlObject(%player);
}



//-----------------------------------------------------------------------------
// Support functions
//-----------------------------------------------------------------------------

function pickSpawnPoint(%tag) 
{
%a=-1;
if (%tag>0) {
  for (%i=0; %i<missioncleanup.getcount(); %i++) {
   if (missioncleanup.getobject(%i).type==%tag*-1) {
     missioncleanup.getobject(%i).radius=5;
     %cspawn[%a++]=missioncleanup.getobject(%i);
     }
   }
   if (%a>=0) {
     %index = getRandom(%a);
     %spawn = %cspawn[%index];
     }
   else
     %spawn=0;
}
else {
echo("default spawn");
   %groupName = "MissionGroup/PlayerDropPoints";
   %group = nameToID(%groupName);

	if (%group != -1) {
		%count = %group.getCount();
		if (%count != 0) {
			%index = getRandom(%count-1);
			%spawn = %group.getObject(%index);
                        }
                }
      }
if (isobject(%spawn)) {
  %trans = %spawn.getTransform();
  %transX = getWord(%trans, 0);
  %transY = getWord(%trans, 1);
  %transZ = getWord(%trans, 2);
  %r = getRandom(%spawn.radius * 10) / 10;
  %ang = getRandom($pi * 2 * 100) / 100;
  %transX += %r * mCos(%ang);
  %transY += %r * mSin(%ang);
  %transXY = %transX @ " " @ %transY;
  %spawnAngle = getRandom($pi * 2 * 100) / 100;
  %returnTrans = %transX  @ " " @ %transY @ " " @ %transZ @ " 0 0 1 " @ %spawnAngle;
  return %returnTrans;
  }
else
   return "0 0 300 1 0 0 0";
}

